module Init (
         side,
         area,
         module Init.Size,
         module Init.SizeDistr,
         module Init.Square,
         module Init.BoardPart,
         InitBoard,
         Name,
         Reason (..),
         Score (..),
         Shape (..),
         Response (..),
         ServerMsg (..),
         PlayerMsg (..)
       ) where


{-

See moodul vtab kokku ja ekspordib kik kataloogi  Init  moodulites
eksporditud nimed. Lisaks defineerib see mned uued algebralised
andmetbid, nende vrtuste teisendamise stringiks ja parsimise stringist.

Muuhulgas on defineeritud algebralised andmetbid serveri- ja
mngijateadete jaoks. Need on realiseeritud nii, et algebralise vrtuse
teisendamisel stringiks saadakse protokollile vastav string ning
protokollile vastav string on heselt parsitav algebraliseks vrtuseks.

Niisiis piisab algebralise vrtusena esitatud teate protokollile vastavaks
stringiks tegemiseks rakendada funktsiooni  show  ning protokollile vastava
teate lugemiseks rakendada funktsiooni  read . Viimane on kasulik ka
testimisel, sest tihti on vrtuste stringiesitust lihtsam ksurealt
kirjutada kui esitust algebralise andmetbi vrtusena.

Ei maksa end hirida lasta sellest, et funktsiooni  read  siin hegi tbi
jaoks ilmutatult ei defineerita. Ta ttab sellegipoolest, sest preldis
defineeritakse  read  funktsiooni  readsPrec  kaudu, mis siin
defineeritakse.

Veel tuleb thele panna, et funktsiooni  read  vi  reads  vi  readsPrec
kasutamiseks vib olla vaja ilmutatult nidata tp, millesse ta peab
stringi parsima. Niteks  read "BGN"  ksurealt annab "Unresolved
overloading" vea, sest ta ei tea, millisesse tpi tahetakse stringi
teisendada,  read "BGN" :: ServerMsg  vi  
(read :: String -> ServerMsg) "BGN"  ttavad ilusasti. Kui konteksti jrgi
on selge, mis tp peab olema, siis pole ilmutatud tbilisa vaja.

-}


import Data.Maybe
import Utils
import Init.Constants
import Init.Size
import Init.SizeDistr
import Init.Square
import Init.BoardPart


{- Algebraliste andmetpide sissetoomine. -}

-- Algseisu esitame lauaosana, kus kik laevad on hes pundis.
type InitBoard
  = BoardPart

-- Nime esitame stringina.
type Name
  = String

-- Partiilpuphjusetp.
data Reason
  = Normal
  | Timeout
  | Violation
  deriving (Eq)

-- Tulemusetp.
data Score
  = Lose
  | Win
  deriving (Eq, Ord, Bounded)

-- Laevakujutp.
data Shape
  = Straight
  | Tetris
  | Arbitrary
  deriving (Eq, Ord, Bounded)

-- Vastusetp.
data Response
  = Mdas
  | Pihtas
  | Phjas
  deriving (Eq, Ord, Bounded)

-- Serveriteatetp.
data ServerMsg
  = Bgn
  | OppShips InitBoard
  | End Reason Score
  | Ext
  | Mov
  | OppNam Name
  | Par SizeDistr Shape
  | Rsp Response
  | Bomb Square
  | Who

-- Mngijateatetp.
data PlayerMsg
  = MyShips InitBoard
  | MyNam Name
  | OppSqr Square


{- Abifunktsioonid. -}

showWithConcat
  :: Show a
  => [a] -> ShowS
showWithConcat
  = flip (foldr ((++) . show))

reasonPairs
  :: [(String , Reason)]
reasonPairs
  = [
     ("normal" , Normal)
    ,
     ("timeout" , Timeout)
    ,
     ("violation" , Violation)
    ]

scorePairs
  :: [(String , Score)]
scorePairs
  = [
     ("1:0" , Win)
    ,
     ("0:1" , Lose)
    ]

shapePairs
  :: [(String , Shape)]
shapePairs
  = [
     ("straight" , Straight)
    ,
     ("tetris" , Tetris)
    ,
     ("arbitrary" , Arbitrary)
    ]

responsePairs
  :: [(String , Response)]
responsePairs
  = [
     ("mdas" , Mdas)
    ,
     ("pihtas" , Pihtas)
    ,
     ("phjas" , Phjas)
    ]

defaultShow
  :: Eq a
  => [(String , a)] -> a -> String
defaultShow ps
  = fromJust . flip lookup (map swap ps)

defaultRead
  :: [(String , a)] -> Int -> ReadS a
defaultRead ps _ s
  = let
      (f , s') = break (' ' ==) s
      s'' = drop 1 s'
      lkp = lookup f ps
    in
    case lkp of
      Just a
        -> [(a , s'')]
      _
        -> []

defaultReadList
  :: Read a
  => ReadS [a]
defaultReadList s
  = let
      (f , s') = break (' ' ==) s
      readList ""
        = [[]]
      readList s
        = do
            (x , t) <- reads s
            map (x :) (readList t)
    in
    map (flip (,) (drop 1 s')) (readList f)


{- Tpide deklareerimine klassi  Show  esindajaks, stringiksteisendus. -}

instance Show Reason where

  show
    = defaultShow reasonPairs


instance Show Score where

  show
    = defaultShow scorePairs


instance Show Size where

  show
    = show . intFromSize

  showList
    = showWithConcat


instance Show SizeDistr where

  showsPrec _
    = showList . sizeListFromDistr


instance Show Shape where

  show
    = defaultShow shapePairs


instance Show Response where

  show
    = defaultShow responsePairs


instance Show Square where

  show sq
    = let
        (q , r) = rowCol sq
      in
      toEnum (q + fromEnum 'a') : show r

  showList
    = showWithConcat


instance Show BoardPart where

  showsPrec n
    = showsPrec n . squareListFromBoardPart


instance Show ServerMsg where

  showsPrec n sm as
    = case sm of
        Bgn
          -> 'B' : 'G' : 'N' : '\n' : as
        OppShips b
          -> 'B' : 'R' : 'D' : ' ' : showsPrec n b ('\n' : as)
        End r s
          -> 'E' : 'N' : 'D' : ' ' :
             showsPrec n r (' ' : showsPrec n s ('\n' : as))
        Ext
          -> 'E' : 'X' : 'T' : '\n' : as
        Mov
          -> 'M' : 'O' : 'V' : '\n' : as
        OppNam s
          -> 'N' : 'A' : 'M' : ' ' : s ++ '\n' : as
        Par ss sh
          -> 'P' : 'A' : 'R' : ' ' : 
             showsPrec n ss (' ' : showsPrec n sh ('\n' : as))
        Rsp r
          -> 'R' : 'S' : 'P' : ' ' : showsPrec n r ('\n' : as)
        Bomb s
          -> 'S' : 'Q' : 'R' : ' ' : showsPrec n s ('\n' : as)
        Who
          -> 'W' : 'H' : 'O' : '\n' : as


instance Show PlayerMsg where

  showsPrec n pm as
    = case pm of
        MyShips b
          -> 'B' : 'R' : 'D' : ' ' : showsPrec n b ('\n' : as)
        MyNam s
          -> 'N' : 'A' : 'M' : ' ' : s ++ '\n' : as
        OppSqr s
          -> 'S' : 'Q' : 'R' : ' ' : showsPrec n s ('\n' : as)


{- Tpide defineerimine klassi  Read  esindajaks, stringistparsimine. -}

instance Read Reason where

  readsPrec
    = defaultRead reasonPairs


instance Read Score where

  readsPrec
    = defaultRead scorePairs


instance Read Size where

  readsPrec n s@ ~(f : s')
    = if null s
         then []
         else case sizeFromInt (countFrom '0' f) of
                Just a
                  -> [(a , s')]
                _
                  -> []

  readList
    = defaultReadList


instance Read SizeDistr where

  readsPrec n s
    = do
        (ss , t) <- readList s
        case distrFromSizeList ss of
          Just sd
            -> [(sd , t)]
          _
            -> []


instance Read Shape where

  readsPrec
    = defaultRead shapePairs


instance Read Response where

  readsPrec
    = defaultRead responsePairs


instance Read Square where

  readsPrec n s
    = let
        (f , s') = splitAt 2 s
      in
      case f of
        [ci, cj]
          -> case squareFromPairInt (countFrom 'a' ci , countFrom '0' cj) of
               Just a
                 -> [(a , s')]
               _
                 -> []
        _
          -> []

  readList
    = defaultReadList


instance Read BoardPart where

  readsPrec _ s
    = do
        (sqs , t) <- readList s
        case boardPartFromSquareList sqs of
          Just b
            -> [(b , t)]
          _
            -> []


instance Read ServerMsg where

  readsPrec _ s
    = let
        (msg , z) = break ('\n' ==) s
        zz = drop 1 z
        (cmd , s') = break (' ' ==) msg
        s'' = drop 1 s'
      in
      case cmd of
        "BGN"
          -> if null s''
                then [(Bgn , zz)]
                else []
        "BRD"
          -> if s'' == s'
                then []
                else do
                       (brd , t) <- reads s''
                       if null t
                          then [(OppShips brd , zz)]
                          else []
        "END"
          -> do
               (rsn , t) <- reads s''
               (sc , u) <- reads t
               if null u
                  then [(End rsn sc , zz)]
                  else []
        "EXT"
          -> if null s''
                then [(Ext , zz)]
                else []
        "MOV"
          -> if null s''
                then [(Mov , zz)]
                else []
        "NAM"
          -> if s'' == s'
                then []
                else [(OppNam s'' , zz)]
        "PAR"
          -> do
               (sizes , t) <- reads s''
               (shape , u) <- reads t
               if null u
                  then [(Par sizes shape , zz)]
                  else []
        "RSP"
          -> do
               (rsp , t) <- reads s''
               if null t
                  then [(Rsp rsp , zz)]
                  else []
        "SQR"
          -> do
               (sq , t) <- reads s''
               if null t
                  then [(Bomb sq , zz)]
                  else []
        "WHO"
          -> if null s''
                then [(Who , zz)]
                else []
        _
          -> []


instance Read PlayerMsg where

  readsPrec _ s
    = let
        (msg , z) = break ('\n' ==) s
        zz = drop 1 z
        (cmd , s') = break (' ' ==) msg
        s'' = drop 1 s'
      in
      case cmd of
        "BRD"
          -> if s'' == s'
                then []
                else do
                       (brd , t) <- reads s''
                       if null t
                          then [(MyShips brd , zz)]
                          else []
        "NAM"
          -> if s'' == s'
                then []
                else [(MyNam s'' , zz)]
        "SQR"
          -> do
               (sq , t) <- reads s''
               if null t
                  then [(OppSqr sq , zz)]
                  else []
        _
          -> []

