module Init.BoardPart (
         BoardPart,
         boardPartFromSquareList,
         squareListFromBoardPart,
         emptyBoard,
         entireBoard,
         insertToBoardPart,
         deleteFromBoardPart,
         mergeBoardParts,
         joinListBoardPart
       ) where


{-

Siin defineeritakse mnguvljakuosatp  BoardPart , mida vib kasutada
laevade, laevagruppide, thimike vms esitamiseks.

-}


import Data.List
import Utils
import Init.Square


newtype BoardPart
  = Board [Square]


-- Kordusteta ruutude listi teisendamine lauaosaks.
boardPartFromSquareList
  :: [Square] -> Maybe BoardPart
boardPartFromSquareList sqs
  = let
      sqs' = sort sqs
    in
    if (checkConsecutive (==) sqs')
       then Nothing
       else Just (Board sqs')

-- Lauaosa esitamine ruutude sorteeritud kordumisteta listina.
squareListFromBoardPart
  :: BoardPart -> [Square]
squareListFromBoardPart (Board sqs)
  = sqs

-- Thi lauaosa.
emptyBoard
  :: BoardPart
emptyBoard
  = Board []

-- Terve mngulaud lauaosana.
entireBoard
  :: BoardPart
entireBoard
  = Board entire

-- Ruudu lisamine lauaossa. 
insertToBoardPart
  :: Square -> BoardPart -> BoardPart
insertToBoardPart sq (Board sqs)
  = Board (insertToSorted sq sqs)

-- Ruudu eemaldamine lauaosast.
deleteFromBoardPart
  :: Square -> BoardPart -> BoardPart
deleteFromBoardPart sq (Board sqs)
  = Board (deleteFromSorted sq sqs)

-- Kahe lauaosa hendamine.
mergeBoardParts
  :: BoardPart -> BoardPart -> BoardPart
mergeBoardParts brd1 brd2
  = Board (merge (squareListFromBoardPart brd1) (squareListFromBoardPart brd2))

-- Lauaosade listi hendamine.
joinListBoardPart
  :: [BoardPart] -> BoardPart
joinListBoardPart
  = Board . joinSorted . map squareListFromBoardPart

