module Init.Square (
         Square,
         squareFromInt,
         squareFromPairInt,
         intFromSquare,
         rowCol
       ) where


{-

Siin defineeritakse ruudutp  Square  ja seonduvad teisendused.

-}


import Init.Constants


newtype Square
  = Square Int
  deriving (Eq, Ord)


-- Arvupaari teisendamine mnguvljaku ruuduks. Arvupaari esimest komponenti
-- vetakse reaindeksina ja teist veeruindeksina, indeksid muutuvad  0-st
-- 9-ni.
squareFromPairInt
  :: (Int , Int) -> Maybe Square
squareFromPairInt (i , j)
  = let
      legal x
        = 0 <= x && x < side
    in
    if legal i && legal j
       then Just (Square (side * i + j))
       else Nothing

-- Mnguvljaku ruudu numbri teisendamine mnguvljaku ruuduks. Ruute
-- nummerdatakse ruudust  a0  mda ridu ruuduni  j9  arvudega  0  kuni  99 .
squareFromInt
  :: Int -> Maybe Square
squareFromInt
  = squareFromPairInt . flip divMod side

-- Ruudu jrgi tema numbri leidmine.
intFromSquare
  :: Square -> Int
intFromSquare (Square n)
  = n

-- Ruudu jrgi tema rea- ja veerunumbri leidmine.
rowCol
  :: Square -> (Int , Int)
rowCol sq
  = intFromSquare sq `divMod` side


{- Ruudutp saab klasside  Bounded  ja  Enum  esindajaks. -}

instance Bounded Square where

  minBound
    = Square 0

  maxBound
    = Square (area - 1)


instance Enum Square where

  toEnum x
    = case squareFromInt x of
        Just s
          -> s
        _
          -> error ("Can't interpret  " ++ show x ++ "  as a square")

  fromEnum
    = intFromSquare

