module Utils where


{-

See moodul sisaldab definitsioone, mida vibolla lheb vaja (minul lks),
kuid mis otseselt antud lesandega ei seondu.

-}


{- ldised utiliidid. -}

-- Ruututstmine.
sqr
  :: Num a
  => a -> a
sqr x
  = x * x

-- Paari komponentide vahetamine.
swap
  :: (a , b) -> (b , a)
swap (x , y)
  = (y , x)


{- Piiratud tpidega seonduvad utiliidid. -}

-- Kigi vrtuste list kasvavas jrjekorras.
entire
  :: (Bounded a, Enum a)
  => [a]
entire
  = [minBound .. maxBound]

-- Kahe arvu laisk miinimum ja maksimum.
minB, maxB
  :: (Ord a, Bounded a)
  => a -> a -> a
minB x y
  = if x == minBound
       then x
       else min x y
maxB x y
  = if x == maxBound
       then x
       else max x y

-- Listi miinimum ja maksimum. Ttavad ka thjal listil.
minimumB, maximumB
  :: (Ord a, Bounded a)
  => [a] -> a
minimumB
  = foldr minB maxBound
maximumB
  = foldr maxB minBound


{- Smbolite ja stringide utiliidid. -}

-- Smbolite arv kooditabelis hest smbolist teiseni.
countFrom
  :: Char -> Char -> Int
countFrom c d
  = fromEnum d - fromEnum c

-- Backspace'i kontroll.
isBsp
  :: Char -> Bool
isBsp
  = (127 ==) . fromEnum

-- ige stringi leidmine, vttes arvesse ka backspace'e.
reduce
  :: String -> String
reduce
  = let
      reduce as (c : cs)
        = if isBsp c
             then reduce (drop 1 as) cs
             else reduce (c : as) cs
      reduce as _
        = reverse as
    in
    reduce []


{- Listidega seonduvad operatsioonid. -}

-- Listi ainsa elemendi leidmine.
singleton [x]
  = Just x
singleton _
  = Nothing

-- heelemendilise listi puhul ainus element, muidu thi.
singleOrNo
  :: [[a]] -> [a]
singleOrNo xs@ ~(z : _)
  = if null xs
       then []
       else z

-- Listi elementide kahekaupa kokkuvtmine ja neile binaarse operatsiooni
-- rakendamine.
map2
  :: (a -> a -> a) -> [a] -> [a]
map2 (*) (x : y : ys)
  = x * y : map2 (*) ys
map2 _   xs
  = xs

-- Listi elementide kokkuarvutamine, kui sulud on asetatud puuviisi.
foldt
  :: (a -> a -> a) -> [a] -> [a]
foldt (*) xs@ (_ : _ : _)
  = foldt (*) (map2 (*) xs)
foldt _   xs
  = xs

-- Kontrollib, kas mingid kaks jrjestikust elementi listis rahuldavad 
-- predikaati.
checkConsecutive
  :: (a -> a -> Bool) -> [a] -> Bool
checkConsecutive p xs
  = or (zipWith p xs (tail xs))

-- Kontrollib, kas listi kik elemendid on vrdsed.
allEq
  :: Eq a
  => [a] -> Bool
allEq
  = not . checkConsecutive (/=)


{- Operatsioonid, mis ttavad eeldusel, et argumentlistid on sorteeritud ja 
   kordumisteta. Nende poolt genereeritud listid rahuldavad samu nudeid. -}

-- Elemendi lisamine.
insertToSorted
  :: Ord a
  => a -> [a] -> [a]
insertToSorted a xs@ (z : zs)
  = case compare a z of
      GT
        -> z : insertToSorted a zs
      EQ
        -> xs
      _
        -> a : xs
insertToSorted a _
  = [a]

-- Elemendi eemaldamine.
deleteFromSorted
  :: Ord a
  => a -> [a] -> [a]
deleteFromSorted a xs@ (z : zs)
  = case compare a z of
      GT
        -> z : deleteFromSorted a zs
      EQ
        -> zs
      _
        -> xs
deleteFromSorted _ _
  = []

-- Kahe listi hendamine.
merge
  :: Ord a
  => [a] -> [a] -> [a]
merge as@ (x : xs) bs@ (y : ys)
  = case compare x y of
      LT
        -> x : merge xs bs
      GT
        -> y : merge as ys
      _
        -> x : merge xs ys
merge []           bs
  = bs
merge as           _
  = as

-- Listide listi hendamine.
joinSorted
  :: Ord a
  => [[a]] -> [a]
joinSorted
  = singleOrNo . foldt merge

-- Kahe listi hisosa leidmine.
intersectSorted
  :: Ord a
  => [a] -> [a] -> [a]
intersectSorted as@ (x : xs) bs@ (y : ys)
  = case compare x y of
      LT
        -> intersectSorted xs bs
      GT
        -> intersectSorted as ys
      _
        -> x : intersectSorted xs ys
intersectSorted _            _
  = []

-- Vahe leidmine.
diffSorted
  :: Ord a
  => [a] -> [a] -> [a]
diffSorted as@ (x : xs) bs@ (y : ys)
  = case compare x y of
      LT
        -> x : diffSorted xs bs
      GT
        -> diffSorted as ys
      _
        -> diffSorted xs ys
diffSorted as           []
  = as
diffSorted _            _
  = []

-- Kontroll, kas element esineb listis.
elemInSorted
  :: Ord a
  => a -> [a] -> Bool
elemInSorted x (z : zs)
  = if x > z
       then elemInSorted x zs
       else x == z
elemInSorted x _
  = False

-- Kontroll, kas esimese listi kik elemendid esinevad teises.
subSorted
  :: Ord a
  => [a] -> [a] -> Bool
subSorted xs ys
  = null (diffSorted xs ys)


{- Listist he elemendi vljavtetega seonduvad operatsioonid. -}

-- ldine konkreetse indeksiga mratud vljavttega seonduv abifunktsioon.
selectC
  :: ([a] -> a -> o) -> Int -> [a] -> o
selectC c n xs@ (z : zs)
  = case compare n 0 of
      GT
        -> selectC (c . (z :)) (n - 1) zs
      EQ
        -> c zs z
      _
        -> c xs (error "selectC: negative index")
selectC c n xs
  = c xs (error "selectC: index too large")

-- Indeksi jrgi vljavtte leidmine.
select
  :: Int -> [a] -> (a , [a])
select
  = selectC (flip (,))

-- Kigi vljavtete listi leidmine.
selections
  :: [a] -> [(a , [a])]
selections xs
  = map (flip select xs) (zipWith const [0 ..] xs)

